@@ -20,6 +20,8 @@ from xarray.core.variable import IndexVariable, Variable
 from xarray.namedarray.utils import is_duck_dask_array
 
 if TYPE_CHECKING:
+    from typing import Self
+
     from numpy.typing import DTypeLike
 
     from xarray.core.dataarray import DataArray
@@ -204,7 +206,7 @@ def _strftime_through_cftimeindex(values, date_format: str):
     values_as_cftimeindex = CFTimeIndex(duck_array_ops.ravel(values))
 
     field_values = values_as_cftimeindex.strftime(date_format)
-    return field_values.values.reshape(values.shape)
+    return field_values.to_numpy().reshape(values.shape)
 
 
 def _strftime_through_series(values, date_format: str):
@@ -213,7 +215,7 @@ def _strftime_through_series(values, date_format: str):
     """
     values_as_series = pd.Series(duck_array_ops.ravel(values), copy=False)
     strs = values_as_series.dt.strftime(date_format)
-    return strs.values.reshape(values.shape)
+    return strs.to_numpy().reshape(values.shape)
 
 
 def _strftime(values, date_format):
@@ -242,7 +244,7 @@ class TimeAccessor(Generic[T_DataArray]):
     def __init__(self, obj: T_DataArray) -> None:
         self._obj = obj
 
-    def _date_field(self, name: str, dtype: DTypeLike) -> T_DataArray:
+    def _date_field(self, name: str, dtype: DTypeLike | None) -> T_DataArray:
         if dtype is None:
             dtype = self._obj.dtype
         result = _get_date_field(_index_or_data(self._obj), name, dtype)
@@ -650,7 +652,7 @@ class TimedeltaAccessor(TimeAccessor[T_DataArray]):
 class CombinedDatetimelikeAccessor(
     DatetimeAccessor[T_DataArray], TimedeltaAccessor[T_DataArray]
 ):
-    def __new__(cls, obj: T_DataArray) -> CombinedDatetimelikeAccessor:
+    def __new__(cls, obj: T_DataArray) -> Self:
         # CombinedDatetimelikeAccessor isn't really instantiated. Instead
         # we need to choose which parent (datetime or timedelta) is
         # appropriate. Since we're checking the dtypes anyway, we'll just
