@@ -4,10 +4,11 @@ import functools
 from typing import Any
 
 import numpy as np
-from pandas.api.types import is_extension_array_dtype
+import pandas as pd
 
-from xarray.core import array_api_compat, npcompat, utils
-from xarray.core.npcompat import HAS_STRING_DTYPE
+from xarray.compat import array_api_compat, npcompat
+from xarray.compat.npcompat import HAS_STRING_DTYPE
+from xarray.core import utils
 
 # Use as a sentinel value to indicate a dtype appropriate NA value.
 NA = utils.ReprObject("<NA>")
@@ -62,7 +63,7 @@ def maybe_promote(dtype: np.dtype) -> tuple[np.dtype, Any]:
     # N.B. these casting rules should match pandas
     dtype_: np.typing.DTypeLike
     fill_value: Any
-    if HAS_STRING_DTYPE and np.issubdtype(dtype, np.dtypes.StringDType()):  # type: ignore[attr-defined]
+    if HAS_STRING_DTYPE and np.issubdtype(dtype, np.dtypes.StringDType()):
         # for now, we always promote string dtypes to object for consistency with existing behavior
         # TODO: refactor this once we have a better way to handle numpy vlen-string dtypes
         dtype_ = object
@@ -208,11 +209,11 @@ def isdtype(dtype, kind: str | tuple[str, ...], xp=None) -> bool:
     if not isinstance(kind, str) and not (
         isinstance(kind, tuple) and all(isinstance(k, str) for k in kind)  # type: ignore[redundant-expr]
     ):
-        raise TypeError(f"kind must be a string or a tuple of strings: {repr(kind)}")
+        raise TypeError(f"kind must be a string or a tuple of strings: {kind!r}")
 
     if isinstance(dtype, np.dtype):
         return npcompat.isdtype(dtype, kind)
-    elif is_extension_array_dtype(dtype):
+    elif pd.api.types.is_extension_array_dtype(dtype):  # noqa: TID251
         # we never want to match pandas extension array dtypes
         return False
     else:
@@ -237,7 +238,7 @@ def preprocess_types(t):
 
 
 def result_type(
-    *arrays_and_dtypes: np.typing.ArrayLike | np.typing.DTypeLike,
+    *arrays_and_dtypes: np.typing.ArrayLike | np.typing.DTypeLike | None,
     xp=None,
 ) -> np.dtype:
     """Like np.result_type, but with type promotion rules matching pandas.
