@@ -37,119 +37,119 @@ if TYPE_CHECKING:
 
 def _dsplot(plotfunc):
     commondoc = """
-    Parameters
-    ----------
+Parameters
+----------
 
-    ds : Dataset
-    x : Hashable or None, optional
-        Variable name for x-axis.
-    y : Hashable or None, optional
-        Variable name for y-axis.
-    u : Hashable or None, optional
-        Variable name for the *u* velocity (in *x* direction).
-        quiver/streamplot plots only.
-    v : Hashable or None, optional
-        Variable name for the *v* velocity (in *y* direction).
-        quiver/streamplot plots only.
-    hue: Hashable or None, optional
-        Variable by which to color scatter points or arrows.
-    hue_style: {'continuous', 'discrete'} or None, optional
-        How to use the ``hue`` variable:
+ds : Dataset
+x : Hashable or None, optional
+    Variable name for x-axis.
+y : Hashable or None, optional
+    Variable name for y-axis.
+u : Hashable or None, optional
+    Variable name for the *u* velocity (in *x* direction).
+    quiver/streamplot plots only.
+v : Hashable or None, optional
+    Variable name for the *v* velocity (in *y* direction).
+    quiver/streamplot plots only.
+hue: Hashable or None, optional
+    Variable by which to color scatter points or arrows.
+hue_style: {'continuous', 'discrete'} or None, optional
+    How to use the ``hue`` variable:
 
-        - ``'continuous'`` -- continuous color scale
-          (default for numeric ``hue`` variables)
-        - ``'discrete'`` -- a color for each unique value, using the default color cycle
-          (default for non-numeric ``hue`` variables)
+    - ``'continuous'`` -- continuous color scale
+        (default for numeric ``hue`` variables)
+    - ``'discrete'`` -- a color for each unique value, using the default color cycle
+        (default for non-numeric ``hue`` variables)
 
-    row : Hashable or None, optional
-        If passed, make row faceted plots on this dimension name.
-    col : Hashable or None, optional
-        If passed, make column faceted plots on this dimension name.
-    col_wrap : int, optional
-        Use together with ``col`` to wrap faceted plots.
-    ax : matplotlib axes object or None, optional
-        If ``None``, use the current axes. Not applicable when using facets.
-    figsize : Iterable[float] or None, optional
-        A tuple (width, height) of the figure in inches.
-        Mutually exclusive with ``size`` and ``ax``.
-    size : scalar, optional
-        If provided, create a new figure for the plot with the given size.
-        Height (in inches) of each plot. See also: ``aspect``.
-    aspect : "auto", "equal", scalar or None, optional
-        Aspect ratio of plot, so that ``aspect * size`` gives the width in
-        inches. Only used if a ``size`` is provided.
-    sharex : bool or None, optional
-        If True all subplots share the same x-axis.
-    sharey : bool or None, optional
-        If True all subplots share the same y-axis.
-    add_guide: bool or None, optional
-        Add a guide that depends on ``hue_style``:
+row : Hashable or None, optional
+    If passed, make row faceted plots on this dimension name.
+col : Hashable or None, optional
+    If passed, make column faceted plots on this dimension name.
+col_wrap : int, optional
+    Use together with ``col`` to wrap faceted plots.
+ax : matplotlib axes object or None, optional
+    If ``None``, use the current axes. Not applicable when using facets.
+figsize : Iterable[float] or None, optional
+    A tuple (width, height) of the figure in inches.
+    Mutually exclusive with ``size`` and ``ax``.
+size : scalar, optional
+    If provided, create a new figure for the plot with the given size.
+    Height (in inches) of each plot. See also: ``aspect``.
+aspect : "auto", "equal", scalar or None, optional
+    Aspect ratio of plot, so that ``aspect * size`` gives the width in
+    inches. Only used if a ``size`` is provided.
+sharex : bool or None, optional
+    If True all subplots share the same x-axis.
+sharey : bool or None, optional
+    If True all subplots share the same y-axis.
+add_guide: bool or None, optional
+    Add a guide that depends on ``hue_style``:
 
-        - ``'continuous'`` -- build a colorbar
-        - ``'discrete'`` -- build a legend
+    - ``'continuous'`` -- build a colorbar
+    - ``'discrete'`` -- build a legend
 
-    subplot_kws : dict or None, optional
-        Dictionary of keyword arguments for Matplotlib subplots
-        (see :py:meth:`matplotlib:matplotlib.figure.Figure.add_subplot`).
-        Only applies to FacetGrid plotting.
-    cbar_kwargs : dict, optional
-        Dictionary of keyword arguments to pass to the colorbar
-        (see :meth:`matplotlib:matplotlib.figure.Figure.colorbar`).
-    cbar_ax : matplotlib axes object, optional
-        Axes in which to draw the colorbar.
-    cmap : matplotlib colormap name or colormap, optional
-        The mapping from data values to color space. Either a
-        Matplotlib colormap name or object. If not provided, this will
-        be either ``'viridis'`` (if the function infers a sequential
-        dataset) or ``'RdBu_r'`` (if the function infers a diverging
-        dataset).
-        See :doc:`Choosing Colormaps in Matplotlib <matplotlib:users/explain/colors/colormaps>`
-        for more information.
+subplot_kws : dict or None, optional
+    Dictionary of keyword arguments for Matplotlib subplots
+    (see :py:meth:`matplotlib:matplotlib.figure.Figure.add_subplot`).
+    Only applies to FacetGrid plotting.
+cbar_kwargs : dict, optional
+    Dictionary of keyword arguments to pass to the colorbar
+    (see :meth:`matplotlib:matplotlib.figure.Figure.colorbar`).
+cbar_ax : matplotlib axes object, optional
+    Axes in which to draw the colorbar.
+cmap : matplotlib colormap name or colormap, optional
+    The mapping from data values to color space. Either a
+    Matplotlib colormap name or object. If not provided, this will
+    be either ``'viridis'`` (if the function infers a sequential
+    dataset) or ``'RdBu_r'`` (if the function infers a diverging
+    dataset).
+    See :doc:`Choosing Colormaps in Matplotlib <matplotlib:users/explain/colors/colormaps>`
+    for more information.
 
-        If *seaborn* is installed, ``cmap`` may also be a
-        `seaborn color palette <https://seaborn.pydata.org/tutorial/color_palettes.html>`_.
-        Note: if ``cmap`` is a seaborn color palette,
-        ``levels`` must also be specified.
-    vmin : float or None, optional
-        Lower value to anchor the colormap, otherwise it is inferred from the
-        data and other keyword arguments. When a diverging dataset is inferred,
-        setting `vmin` or `vmax` will fix the other by symmetry around
-        ``center``. Setting both values prevents use of a diverging colormap.
-        If discrete levels are provided as an explicit list, both of these
-        values are ignored.
-    vmax : float or None, optional
-        Upper value to anchor the colormap, otherwise it is inferred from the
-        data and other keyword arguments. When a diverging dataset is inferred,
-        setting `vmin` or `vmax` will fix the other by symmetry around
-        ``center``. Setting both values prevents use of a diverging colormap.
-        If discrete levels are provided as an explicit list, both of these
-        values are ignored.
-    norm : matplotlib.colors.Normalize, optional
-        If ``norm`` has ``vmin`` or ``vmax`` specified, the corresponding
-        kwarg must be ``None``.
-    infer_intervals: bool | None
-        If True the intervals are inferred.
-    center : float, optional
-        The value at which to center the colormap. Passing this value implies
-        use of a diverging colormap. Setting it to ``False`` prevents use of a
-        diverging colormap.
-    robust : bool, optional
-        If ``True`` and ``vmin`` or ``vmax`` are absent, the colormap range is
-        computed with 2nd and 98th percentiles instead of the extreme values.
-    colors : str or array-like of color-like, optional
-        A single color or a list of colors. The ``levels`` argument
-        is required.
-    extend : {'neither', 'both', 'min', 'max'}, optional
-        How to draw arrows extending the colorbar beyond its limits. If not
-        provided, ``extend`` is inferred from ``vmin``, ``vmax`` and the data limits.
-    levels : int or array-like, optional
-        Split the colormap (``cmap``) into discrete color intervals. If an integer
-        is provided, "nice" levels are chosen based on the data range: this can
-        imply that the final number of levels is not exactly the expected one.
-        Setting ``vmin`` and/or ``vmax`` with ``levels=N`` is equivalent to
-        setting ``levels=np.linspace(vmin, vmax, N)``.
-    **kwargs : optional
-        Additional keyword arguments to wrapped Matplotlib function.
+    If *seaborn* is installed, ``cmap`` may also be a
+    `seaborn color palette <https://seaborn.pydata.org/tutorial/color_palettes.html>`_.
+    Note: if ``cmap`` is a seaborn color palette,
+    ``levels`` must also be specified.
+vmin : float or None, optional
+    Lower value to anchor the colormap, otherwise it is inferred from the
+    data and other keyword arguments. When a diverging dataset is inferred,
+    setting `vmin` or `vmax` will fix the other by symmetry around
+    ``center``. Setting both values prevents use of a diverging colormap.
+    If discrete levels are provided as an explicit list, both of these
+    values are ignored.
+vmax : float or None, optional
+    Upper value to anchor the colormap, otherwise it is inferred from the
+    data and other keyword arguments. When a diverging dataset is inferred,
+    setting `vmin` or `vmax` will fix the other by symmetry around
+    ``center``. Setting both values prevents use of a diverging colormap.
+    If discrete levels are provided as an explicit list, both of these
+    values are ignored.
+norm : matplotlib.colors.Normalize, optional
+    If ``norm`` has ``vmin`` or ``vmax`` specified, the corresponding
+    kwarg must be ``None``.
+infer_intervals: bool | None
+    If True the intervals are inferred.
+center : float, optional
+    The value at which to center the colormap. Passing this value implies
+    use of a diverging colormap. Setting it to ``False`` prevents use of a
+    diverging colormap.
+robust : bool, optional
+    If ``True`` and ``vmin`` or ``vmax`` are absent, the colormap range is
+    computed with 2nd and 98th percentiles instead of the extreme values.
+colors : str or array-like of color-like, optional
+    A single color or a list of colors. The ``levels`` argument
+    is required.
+extend : {'neither', 'both', 'min', 'max'}, optional
+    How to draw arrows extending the colorbar beyond its limits. If not
+    provided, ``extend`` is inferred from ``vmin``, ``vmax`` and the data limits.
+levels : int or array-like, optional
+    Split the colormap (``cmap``) into discrete color intervals. If an integer
+    is provided, "nice" levels are chosen based on the data range: this can
+    imply that the final number of levels is not exactly the expected one.
+    Setting ``vmin`` and/or ``vmax`` with ``levels=N`` is equivalent to
+    setting ``levels=np.linspace(vmin, vmax, N)``.
+**kwargs : optional
+    Additional keyword arguments to wrapped Matplotlib function.
     """
 
     # Build on the original docstring
@@ -630,6 +630,8 @@ def streamplot(
     cmap_params = kwargs.pop("cmap_params")
 
     if hue:
+        if xdim is not None and ydim is not None:
+            ds[hue] = ds[hue].transpose(ydim, xdim)
         kwargs["color"] = ds[hue].values
 
         # TODO: Fix this by always returning a norm with vmin, vmax in cmap_params
