@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import warnings
-from itertools import product
+from itertools import product, starmap
 from typing import TYPE_CHECKING, Literal
 
 import numpy as np
@@ -274,7 +274,7 @@ def test_to_offset_annual(month_label, month_int, multiple, offset_str):
     freq = offset_str
     offset_type = _ANNUAL_OFFSET_TYPES[offset_str]
     if month_label:
-        freq = "-".join([freq, month_label])
+        freq = f"{freq}-{month_label}"
     if multiple:
         freq = f"{multiple}{freq}"
     result = to_offset(freq)
@@ -303,7 +303,7 @@ def test_to_offset_quarter(month_label, month_int, multiple, offset_str):
     freq = offset_str
     offset_type = _QUARTER_OFFSET_TYPES[offset_str]
     if month_label:
-        freq = "-".join([freq, month_label])
+        freq = f"{freq}-{month_label}"
     if multiple:
         freq = f"{multiple}{freq}"
     result = to_offset(freq)
@@ -313,18 +313,16 @@ def test_to_offset_quarter(month_label, month_int, multiple, offset_str):
     elif multiple:
         if month_int:
             expected = offset_type(n=multiple)
-        else:
-            if offset_type == QuarterBegin:
-                expected = offset_type(n=multiple, month=1)
-            elif offset_type == QuarterEnd:
-                expected = offset_type(n=multiple, month=12)
+        elif offset_type == QuarterBegin:
+            expected = offset_type(n=multiple, month=1)
+        elif offset_type == QuarterEnd:
+            expected = offset_type(n=multiple, month=12)
     elif month_int:
         expected = offset_type(month=month_int)
-    else:
-        if offset_type == QuarterBegin:
-            expected = offset_type(month=1)
-        elif offset_type == QuarterEnd:
-            expected = offset_type(month=12)
+    elif offset_type == QuarterBegin:
+        expected = offset_type(month=1)
+    elif offset_type == QuarterEnd:
+        expected = offset_type(month=12)
     assert result == expected
 
 
@@ -447,7 +445,6 @@ _MUL_TESTS = [
     (Second(), 3, Second(n=3)),
     (Millisecond(), 3, Millisecond(n=3)),
     (Microsecond(), 3, Microsecond(n=3)),
-    (Day(), 0.5, Hour(n=12)),
     (Hour(), 0.5, Minute(n=30)),
     (Hour(), -0.5, Minute(n=-30)),
     (Minute(), 0.5, Second(n=30)),
@@ -474,7 +471,15 @@ def test_mul_float_multiple_next_higher_resolution():
 
 @pytest.mark.parametrize(
     "offset",
-    [YearBegin(), YearEnd(), QuarterBegin(), QuarterEnd(), MonthBegin(), MonthEnd()],
+    [
+        YearBegin(),
+        YearEnd(),
+        QuarterBegin(),
+        QuarterEnd(),
+        MonthBegin(),
+        MonthEnd(),
+        Day(),
+    ],
     ids=_id_func,
 )
 def test_nonTick_offset_multiplied_float_error(offset):
@@ -536,6 +541,20 @@ def test_add_sub_monthly(offset, expected_date_args, calendar):
     assert result == expected
 
 
+def test_add_daily_offsets() -> None:
+    offset = Day(n=2)
+    expected = Day(n=4)
+    result = offset + offset
+    assert result == expected
+
+
+def test_subtract_daily_offsets() -> None:
+    offset = Day(n=2)
+    expected = Day(n=0)
+    result = offset - offset
+    assert result == expected
+
+
 @pytest.mark.parametrize(("offset", "expected_date_args"), _ADD_TESTS, ids=_id_func)
 def test_radd_sub_monthly(offset, expected_date_args, calendar):
     date_type = get_date_type(calendar)
@@ -1222,7 +1241,7 @@ def test_cftime_range(
     start, end, periods, freq, inclusive, normalize, calendar, expected_date_args
 ):
     date_type = get_date_type(calendar)
-    expected_dates = [date_type(*args) for args in expected_date_args]
+    expected_dates = list(starmap(date_type, expected_date_args))
 
     if isinstance(start, tuple):
         start = date_type(*start)
@@ -1279,7 +1298,7 @@ def test_invalid_date_range_cftime_inputs(
     end: str | None,
     periods: int | None,
     freq: str | None,
-    inclusive: Literal["up", None],
+    inclusive: Literal["up"] | None,
 ) -> None:
     with pytest.raises(ValueError):
         date_range(start, end, periods, freq, inclusive=inclusive, use_cftime=True)  # type: ignore[arg-type]
@@ -1490,7 +1509,7 @@ def test_date_range_like_errors():
 
     with pytest.raises(
         ValueError,
-        match="`date_range_like` was unable to generate a range as the source frequency was not inferable.",
+        match=r"`date_range_like` was unable to generate a range as the source frequency was not inferable.",
     ):
         date_range_like(src, "gregorian")
 
@@ -1503,14 +1522,14 @@ def test_date_range_like_errors():
     )
     with pytest.raises(
         ValueError,
-        match="'source' must be a 1D array of datetime objects for inferring its range.",
+        match=r"'source' must be a 1D array of datetime objects for inferring its range.",
     ):
         date_range_like(src, "noleap")
 
     da = DataArray([1, 2, 3, 4], dims=("time",))
     with pytest.raises(
         ValueError,
-        match="'source' must be a 1D array of datetime objects for inferring its range.",
+        match=r"'source' must be a 1D array of datetime objects for inferring its range.",
     ):
         date_range_like(da, "noleap")
 
