@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 from collections.abc import Callable
+from contextlib import AbstractContextManager, nullcontext
 from copy import deepcopy
 from typing import TYPE_CHECKING, Any, Literal
 
@@ -8,8 +9,8 @@ import numpy as np
 import pandas as pd
 import pytest
 
-from xarray import DataArray, Dataset, Variable, concat
-from xarray.core import dtypes
+from xarray import AlignmentError, DataArray, Dataset, Variable, concat, set_options
+from xarray.core import dtypes, types
 from xarray.core.coordinates import Coordinates
 from xarray.core.indexes import PandasIndex
 from xarray.structure import merge
@@ -21,7 +22,9 @@ from xarray.tests import (
     assert_equal,
     assert_identical,
     requires_dask,
+    requires_pyarrow,
 )
+from xarray.tests.indexes import XYIndex
 from xarray.tests.test_dataset import create_test_data
 
 if TYPE_CHECKING:
@@ -43,7 +46,7 @@ def create_concat_datasets(
                 ["x", "y", "day"],
                 rng.standard_normal(size=(1, 4, 2)),
             )
-            data_vars = {v: data_tuple for v in variables}
+            data_vars = dict.fromkeys(variables, data_tuple)
             result.append(
                 Dataset(
                     data_vars=data_vars,
@@ -59,7 +62,7 @@ def create_concat_datasets(
                 ["x", "y"],
                 rng.standard_normal(size=(1, 4)),
             )
-            data_vars = {v: data_tuple for v in variables}
+            data_vars = dict.fromkeys(variables, data_tuple)
             result.append(
                 Dataset(
                     data_vars=data_vars,
@@ -133,9 +136,9 @@ def test_concat_compat() -> None:
     for var in ["has_x", "no_x_y"]:
         assert "y" not in result[var].dims and "y" not in result[var].coords
     with pytest.raises(ValueError, match=r"'q' not present in all datasets"):
-        concat([ds1, ds2], dim="q")
+        concat([ds1, ds2], dim="q", data_vars="all", join="outer")
     with pytest.raises(ValueError, match=r"'q' not present in all datasets"):
-        concat([ds2, ds1], dim="q")
+        concat([ds2, ds1], dim="q", data_vars="all", join="outer")
 
 
 def test_concat_missing_var() -> None:
@@ -154,19 +157,20 @@ def test_concat_missing_var() -> None:
     assert_identical(actual, expected)
 
 
-def test_concat_categorical() -> None:
+@pytest.mark.parametrize("var", ["var4", pytest.param("var5", marks=requires_pyarrow)])
+def test_concat_extension_array(var) -> None:
     data1 = create_test_data(use_extension_array=True)
     data2 = create_test_data(use_extension_array=True)
     concatenated = concat([data1, data2], dim="dim1")
-    assert (
-        concatenated["var4"]
-        == type(data2["var4"].variable.data.array)._concat_same_type(
+    assert pd.Series(
+        concatenated[var]
+        == type(data2[var].variable.data)._concat_same_type(
             [
-                data1["var4"].variable.data.array,
-                data2["var4"].variable.data.array,
+                data1[var].variable.data,
+                data2[var].variable.data,
             ]
         )
-    ).all()
+    ).all()  # need to wrap in series because pyarrow bool does not support `all`
 
 
 def test_concat_missing_multiple_consecutive_var() -> None:
@@ -214,8 +218,12 @@ def test_concat_second_empty() -> None:
     actual = concat([ds1, ds2], dim="y", coords="all")
     assert_identical(actual, expected)
 
+
+def test_concat_second_empty_with_scalar_data_var_only_on_first() -> None:
     # Check concatenating scalar data_var only present in ds1
-    ds1["b"] = 0.1
+    ds1 = Dataset(data_vars={"a": ("y", [0.1]), "b": 0.1}, coords={"x": 0.1})
+    ds2 = Dataset(coords={"x": 0.1})
+
     expected = Dataset(
         data_vars={"a": ("y", [0.1, np.nan]), "b": ("y", [0.1, np.nan])},
         coords={"x": ("y", [0.1, 0.1])},
@@ -226,7 +234,9 @@ def test_concat_second_empty() -> None:
     expected = Dataset(
         data_vars={"a": ("y", [0.1, np.nan]), "b": 0.1}, coords={"x": 0.1}
     )
-    actual = concat([ds1, ds2], dim="y", coords="different", data_vars="different")
+    actual = concat(
+        [ds1, ds2], dim="y", coords="different", data_vars="different", compat="equals"
+    )
     assert_identical(actual, expected)
 
 
@@ -263,10 +273,13 @@ def test_concat_multiple_datasets_missing_vars(include_day: bool) -> None:
         "cloud_cover",
     ]
 
+    # must specify if concat_dim='day' is not part of the vars
+    kwargs = {"data_vars": "all"} if not include_day else {}
+
     datasets = create_concat_datasets(
         len(vars_to_drop), seed=123, include_day=include_day
     )
-    expected = concat(datasets, dim="day")
+    expected = concat(datasets, dim="day", **kwargs)  # type: ignore[call-overload]
 
     for i, name in enumerate(vars_to_drop):
         if include_day:
@@ -279,8 +292,7 @@ def test_concat_multiple_datasets_missing_vars(include_day: bool) -> None:
         ds.drop_vars(varname)
         for ds, varname in zip(datasets, vars_to_drop, strict=True)
     ]
-
-    actual = concat(datasets, dim="day")
+    actual = concat(datasets, dim="day", **kwargs)  # type: ignore[call-overload]
 
     assert list(actual.data_vars.keys()) == [
         "pressure",
@@ -490,8 +502,8 @@ class TestConcatDataset:
         "dim,data", [["dim1", True], ["dim2", False]], indirect=["data"]
     )
     def test_concat_simple(self, data: Dataset, dim, coords) -> None:
-        datasets = [g for _, g in data.groupby(dim, squeeze=False)]
-        assert_identical(data, concat(datasets, dim, coords=coords))
+        datasets = [g for _, g in data.groupby(dim)]
+        assert_identical(data, concat(datasets, dim, coords=coords, compat="equals"))
 
     def test_concat_merge_variables_present_in_some_datasets(
         self, data: Dataset
@@ -512,7 +524,7 @@ class TestConcatDataset:
         assert_identical(expected, actual)
 
         # expand foo
-        actual = concat([data0, data1], "dim1")
+        actual = concat([data0, data1], "dim1", data_vars="all")
         foo = np.ones((8, 10), dtype=data1.foo.dtype) * np.nan
         foo[3:] = data1.foo.values[None, ...]
         expected = data.copy().assign(foo=(["dim1", "bar"], foo))
@@ -534,9 +546,11 @@ class TestConcatDataset:
         data = data.copy(deep=True)
         # make sure the coords argument behaves as expected
         data.coords["extra"] = ("dim4", np.arange(3))
-        datasets = [g.squeeze() for _, g in data.groupby(dim, squeeze=False)]
+        datasets = [g for _, g in data.groupby(dim)]
 
-        actual = concat(datasets, data[dim], coords=coords)
+        actual = concat(
+            datasets, data[dim], coords=coords, data_vars="all", compat="equals"
+        )
         if coords == "all":
             expected = np.array([data["extra"].values for _ in range(data.sizes[dim])])
             assert_array_equal(actual["extra"].values, expected)
@@ -568,41 +582,72 @@ class TestConcatDataset:
         actual = concat(objs, dim="x", data_vars="minimal")
         assert_identical(data, actual)
 
-    def test_concat_data_vars(self) -> None:
+    @pytest.mark.parametrize("data_vars", ["minimal", "different", "all", [], ["foo"]])
+    def test_concat_data_vars(self, data_vars) -> None:
         data = Dataset({"foo": ("x", np.random.randn(10))})
         objs: list[Dataset] = [data.isel(x=slice(5)), data.isel(x=slice(5, None))]
-        for data_vars in ["minimal", "different", "all", [], ["foo"]]:
-            actual = concat(objs, dim="x", data_vars=data_vars)
-            assert_identical(data, actual)
+        actual = concat(objs, dim="x", data_vars=data_vars, compat="equals")
+        assert_identical(data, actual)
 
-    def test_concat_coords(self):
-        # TODO: annotating this func fails
+    @pytest.mark.parametrize("coords", ["different", "all", ["c"]])
+    def test_concat_coords(self, coords) -> None:
         data = Dataset({"foo": ("x", np.random.randn(10))})
         expected = data.assign_coords(c=("x", [0] * 5 + [1] * 5))
         objs = [
             data.isel(x=slice(5)).assign_coords(c=0),
             data.isel(x=slice(5, None)).assign_coords(c=1),
         ]
-        for coords in ["different", "all", ["c"]]:
+        if coords == "different":
+            actual = concat(objs, dim="x", coords=coords, compat="equals")
+        else:
             actual = concat(objs, dim="x", coords=coords)
-            assert_identical(expected, actual)
-        for coords in ["minimal", []]:
-            with pytest.raises(merge.MergeError, match="conflicting values"):
-                concat(objs, dim="x", coords=coords)
+        assert_identical(expected, actual)
 
-    def test_concat_constant_index(self):
-        # TODO: annotating this func fails
+    @pytest.mark.parametrize("coords", ["minimal", []])
+    def test_concat_coords_raises_merge_error(self, coords) -> None:
+        data = Dataset({"foo": ("x", np.random.randn(10))})
+        objs = [
+            data.isel(x=slice(5)).assign_coords(c=0),
+            data.isel(x=slice(5, None)).assign_coords(c=1),
+        ]
+        with pytest.raises(merge.MergeError, match="conflicting values"):
+            concat(objs, dim="x", coords=coords, compat="equals")
+
+    @pytest.mark.parametrize("data_vars", ["different", "all", ["foo"]])
+    def test_concat_constant_index(self, data_vars) -> None:
         # GH425
         ds1 = Dataset({"foo": 1.5}, {"y": 1})
         ds2 = Dataset({"foo": 2.5}, {"y": 1})
         expected = Dataset({"foo": ("y", [1.5, 2.5]), "y": [1, 1]})
-        for mode in ["different", "all", ["foo"]]:
-            actual = concat([ds1, ds2], "y", data_vars=mode)
-            assert_identical(expected, actual)
-        with pytest.raises(merge.MergeError, match="conflicting values"):
-            # previously dim="y", and raised error which makes no sense.
-            # "foo" has dimension "y" so minimal should concatenate it?
-            concat([ds1, ds2], "new_dim", data_vars="minimal")
+        if data_vars == "different":
+            actual = concat([ds1, ds2], "y", data_vars=data_vars, compat="equals")
+        else:
+            actual = concat([ds1, ds2], "y", data_vars=data_vars)
+        assert_identical(expected, actual)
+
+    def test_concat_constant_index_None(self) -> None:
+        ds1 = Dataset({"foo": 1.5}, {"y": 1})
+        ds2 = Dataset({"foo": 2.5}, {"y": 1})
+        actual = concat([ds1, ds2], "new_dim", data_vars=None, compat="equals")
+        expected = Dataset(
+            {"foo": ("new_dim", [1.5, 2.5])},
+            coords={"y": 1},
+        )
+        assert_identical(actual, expected)
+
+    def test_concat_constant_index_minimal(self) -> None:
+        ds1 = Dataset({"foo": 1.5}, {"y": 1})
+        ds2 = Dataset({"foo": 2.5}, {"y": 1})
+
+        with set_options(use_new_combine_kwarg_defaults=False):
+            with pytest.raises(merge.MergeError, match="conflicting values"):
+                concat([ds1, ds2], dim="new_dim", data_vars="minimal")
+
+        with set_options(use_new_combine_kwarg_defaults=True):
+            with pytest.raises(
+                ValueError, match="data_vars='minimal' and coords='minimal'"
+            ):
+                concat([ds1, ds2], dim="new_dim", data_vars="minimal")
 
     def test_concat_size0(self) -> None:
         data = create_test_data()
@@ -616,7 +661,7 @@ class TestConcatDataset:
     def test_concat_autoalign(self) -> None:
         ds1 = Dataset({"foo": DataArray([1, 2], coords=[("x", [1, 2])])})
         ds2 = Dataset({"foo": DataArray([1, 2], coords=[("x", [1, 3])])})
-        actual = concat([ds1, ds2], "y")
+        actual = concat([ds1, ds2], "y", data_vars="all", join="outer")
         expected = Dataset(
             {
                 "foo": DataArray(
@@ -628,8 +673,7 @@ class TestConcatDataset:
         )
         assert_identical(expected, actual)
 
-    def test_concat_errors(self):
-        # TODO: annotating this func fails
+    def test_concat_errors(self) -> None:
         data = create_test_data()
         split_data = [data.isel(dim1=slice(3)), data.isel(dim1=slice(3, None))]
 
@@ -659,7 +703,7 @@ class TestConcatDataset:
         assert_identical(data, concat([data0, data1], "dim1", compat="equals"))
 
         with pytest.raises(ValueError, match=r"compat.* invalid"):
-            concat(split_data, "dim1", compat="foobar")
+            concat(split_data, "dim1", compat="foobar")  # type: ignore[call-overload]
 
         with pytest.raises(ValueError, match=r"compat.* invalid"):
             concat(split_data, "dim1", compat="minimal")
@@ -705,9 +749,9 @@ class TestConcatDataset:
         with pytest.raises(ValueError, match=r"cannot align.*exact.*dimensions.*'y'"):
             actual = concat([ds1, ds2], join="exact", dim="x")
 
-        for join in expected:
+        for join, expected_item in expected.items():
             actual = concat([ds1, ds2], join=join, dim="x")
-            assert_equal(actual, expected[join])
+            assert_equal(actual, expected_item)
 
         # regression test for #3681
         actual = concat(
@@ -844,8 +888,7 @@ class TestConcatDataset:
 
             assert_identical(actual, expected)
 
-    def test_concat_promote_shape(self) -> None:
-        # mixed dims within variables
+    def test_concat_promote_shape_with_mixed_dims_within_variables(self) -> None:
         objs = [Dataset({}, {"x": 0}), Dataset({"x": [1]})]
         actual = concat(objs, "x")
         expected = Dataset({"x": [0, 1]})
@@ -855,25 +898,30 @@ class TestConcatDataset:
         actual = concat(objs, "x")
         assert_identical(actual, expected)
 
-        # mixed dims between variables
+    def test_concat_promote_shape_with_mixed_dims_between_variables(self) -> None:
         objs = [Dataset({"x": [2], "y": 3}), Dataset({"x": [4], "y": 5})]
-        actual = concat(objs, "x")
+        actual = concat(objs, "x", data_vars="all")
         expected = Dataset({"x": [2, 4], "y": ("x", [3, 5])})
         assert_identical(actual, expected)
 
-        # mixed dims in coord variable
+    def test_concat_promote_shape_with_mixed_dims_in_coord_variable(self) -> None:
         objs = [Dataset({"x": [0]}, {"y": -1}), Dataset({"x": [1]}, {"y": ("x", [-2])})]
         actual = concat(objs, "x")
         expected = Dataset({"x": [0, 1]}, {"y": ("x", [-1, -2])})
         assert_identical(actual, expected)
 
-        # scalars with mixed lengths along concat dim -- values should repeat
+    def test_concat_promote_shape_for_scalars_with_mixed_lengths_along_concat_dim(
+        self,
+    ) -> None:
+        # values should repeat
         objs = [Dataset({"x": [0]}, {"y": -1}), Dataset({"x": [1, 2]}, {"y": -2})]
-        actual = concat(objs, "x")
+        actual = concat(objs, "x", coords="different", compat="equals")
         expected = Dataset({"x": [0, 1, 2]}, {"y": ("x", [-1, -2, -2])})
         assert_identical(actual, expected)
+        actual = concat(objs, "x", coords="all")
+        assert_identical(actual, expected)
 
-        # broadcast 1d x 1d -> 2d
+    def test_concat_promote_shape_broadcast_1d_x_1d_goes_to_2d(self) -> None:
         objs = [
             Dataset({"z": ("x", [-1])}, {"x": [0], "y": [0]}),
             Dataset({"z": ("y", [1])}, {"x": [1], "y": [0]}),
@@ -882,6 +930,7 @@ class TestConcatDataset:
         expected = Dataset({"z": (("x", "y"), [[-1], [1]])}, {"x": [0, 1], "y": [0]})
         assert_identical(actual, expected)
 
+    def test_concat_promote_shape_with_scalar_coordinates(self) -> None:
         # regression GH6384
         objs = [
             Dataset({}, {"x": pd.Interval(-1, 0, closed="right")}),
@@ -898,6 +947,7 @@ class TestConcatDataset:
         )
         assert_identical(actual, expected)
 
+    def test_concat_promote_shape_with_coordinates_of_particular_dtypes(self) -> None:
         # regression GH6416 (coord dtype) and GH6434
         time_data1 = np.array(["2022-01-01", "2022-02-01"], dtype="datetime64[ns]")
         time_data2 = np.array("2022-03-01", dtype="datetime64[ns]")
@@ -931,14 +981,14 @@ class TestConcatDataset:
         objs = [Dataset({"x": 0}), Dataset({"x": 1})]
         coord = Variable("y", [3, 4], attrs={"foo": "bar"})
         expected = Dataset({"x": ("y", [0, 1]), "y": coord})
-        actual = concat(objs, coord)
+        actual = concat(objs, coord, data_vars="all")
         assert_identical(actual, expected)
 
     def test_concat_dim_is_dataarray(self) -> None:
         objs = [Dataset({"x": 0}), Dataset({"x": 1})]
         coord = DataArray([3, 4], dims="y", attrs={"foo": "bar"})
         expected = Dataset({"x": ("y", [0, 1]), "y": coord})
-        actual = concat(objs, coord)
+        actual = concat(objs, coord, data_vars="all")
         assert_identical(actual, expected)
 
     def test_concat_multiindex(self) -> None:
@@ -965,8 +1015,8 @@ class TestConcatDataset:
     @pytest.mark.parametrize("fill_value", [dtypes.NA, 2, 2.0, {"a": 2, "b": 1}])
     def test_concat_fill_value(self, fill_value) -> None:
         datasets = [
-            Dataset({"a": ("x", [2, 3]), "b": ("x", [-2, 1]), "x": [1, 2]}),
-            Dataset({"a": ("x", [1, 2]), "b": ("x", [3, -1]), "x": [0, 1]}),
+            Dataset({"a": ("x", [2, 3]), "b": ("x", [-2, 1])}, {"x": [1, 2]}),
+            Dataset({"a": ("x", [1, 2]), "b": ("x", [3, -1])}, {"x": [0, 1]}),
         ]
         if fill_value == dtypes.NA:
             # if we supply the default, we expect the missing value for a
@@ -984,7 +1034,9 @@ class TestConcatDataset:
             },
             {"x": [0, 1, 2]},
         )
-        actual = concat(datasets, dim="t", fill_value=fill_value)
+        actual = concat(
+            datasets, dim="t", fill_value=fill_value, data_vars="all", join="outer"
+        )
         assert_identical(actual, expected)
 
     @pytest.mark.parametrize("dtype", [str, bytes])
@@ -1006,7 +1058,7 @@ class TestConcatDataset:
                 "x2": np.array(["c", "d"], dtype=dtype),
             }
         )
-        actual = concat([da1, da2], dim=dim)
+        actual = concat([da1, da2], dim=dim, join="outer")
 
         assert np.issubdtype(actual.x2.dtype, dtype)
 
@@ -1031,7 +1083,7 @@ class TestConcatDataset:
         assert combined.indexes == {}
 
         # should not raise on stack
-        combined = concat(datasets, dim="z")
+        combined = concat(datasets, dim="z", data_vars="all")
         assert combined["a"].shape == (2, 3, 3)
         assert combined["a"].dims == ("z", "x", "y")
 
@@ -1096,11 +1148,15 @@ class TestConcatDataArray:
         stacked = concat(grouped, pd.Index(ds["x"], name="x"))
         assert_identical(foo, stacked)
 
-        actual2 = concat([foo[0], foo[1]], pd.Index([0, 1])).reset_coords(drop=True)
+        actual2 = concat(
+            [foo.isel(x=0), foo.isel(x=1)], pd.Index([0, 1]), coords="all"
+        ).reset_coords(drop=True)
         expected = foo[:2].rename({"x": "concat_dim"})
         assert_identical(expected, actual2)
 
-        actual3 = concat([foo[0], foo[1]], [0, 1]).reset_coords(drop=True)
+        actual3 = concat(
+            [foo.isel(x=0), foo.isel(x=1)], [0, 1], coords="all"
+        ).reset_coords(drop=True)
         expected = foo[:2].rename({"x": "concat_dim"})
         assert_identical(expected, actual3)
 
@@ -1108,7 +1164,7 @@ class TestConcatDataArray:
             concat([foo, bar], dim="w", compat="identical")
 
         with pytest.raises(ValueError, match=r"not a valid argument"):
-            concat([foo, bar], dim="w", data_vars="minimal")
+            concat([foo, bar], dim="w", data_vars="different")
 
     def test_concat_encoding(self) -> None:
         # Regression test for GH1297
@@ -1182,7 +1238,7 @@ class TestConcatDataArray:
             dims=["y", "x"],
             coords={"x": [1, 2, 3]},
         )
-        actual = concat((foo, bar), dim="y", fill_value=fill_value)
+        actual = concat((foo, bar), dim="y", fill_value=fill_value, join="outer")
         assert_identical(actual, expected)
 
     def test_concat_join_kwarg(self) -> None:
@@ -1217,9 +1273,9 @@ class TestConcatDataArray:
         with pytest.raises(ValueError, match=r"cannot align.*exact.*dimensions.*'y'"):
             actual = concat([ds1, ds2], join="exact", dim="x")
 
-        for join in expected:
+        for join, expected_item in expected.items():
             actual = concat([ds1, ds2], join=join, dim="x")
-            assert_equal(actual, expected[join].to_dataarray())
+            assert_equal(actual, expected_item.to_dataarray())
 
     def test_concat_combine_attrs_kwarg(self) -> None:
         da1 = DataArray([0], coords=[("x", [0])], attrs={"b": 42})
@@ -1241,9 +1297,9 @@ class TestConcatDataArray:
             da3.attrs["b"] = 44
             actual = concat([da1, da3], dim="x", combine_attrs="no_conflicts")
 
-        for combine_attrs in expected:
+        for combine_attrs, expected_item in expected.items():
             actual = concat([da1, da2], dim="x", combine_attrs=combine_attrs)
-            assert_identical(actual, expected[combine_attrs])
+            assert_identical(actual, expected_item)
 
     @pytest.mark.parametrize("dtype", [str, bytes])
     @pytest.mark.parametrize("dim", ["x1", "x2"])
@@ -1260,7 +1316,7 @@ class TestConcatDataArray:
             dims=["x1", "x2"],
             coords={"x1": np.array([1, 2]), "x2": np.array(["c", "d"], dtype=dtype)},
         )
-        actual = concat([da1, da2], dim=dim)
+        actual = concat([da1, da2], dim=dim, join="outer")
 
         assert np.issubdtype(actual.x2.dtype, dtype)
 
@@ -1285,16 +1341,17 @@ def test_concat_attrs_first_variable(attr1, attr2) -> None:
     assert concat_attrs == attr1
 
 
-def test_concat_merge_single_non_dim_coord():
-    # TODO: annotating this func fails
+def test_concat_merge_single_non_dim_coord() -> None:
     da1 = DataArray([1, 2, 3], dims="x", coords={"x": [1, 2, 3], "y": 1})
     da2 = DataArray([4, 5, 6], dims="x", coords={"x": [4, 5, 6]})
 
     expected = DataArray(range(1, 7), dims="x", coords={"x": range(1, 7), "y": 1})
 
-    for coords in ["different", "minimal"]:
-        actual = concat([da1, da2], "x", coords=coords)
-        assert_identical(actual, expected)
+    actual = concat([da1, da2], "x", coords="minimal", compat="override")
+    assert_identical(actual, expected)
+
+    actual = concat([da1, da2], "x", coords="different", compat="equals")
+    assert_identical(actual, expected)
 
     with pytest.raises(ValueError, match=r"'y' not present in all datasets."):
         concat([da1, da2], dim="x", coords="all")
@@ -1302,9 +1359,12 @@ def test_concat_merge_single_non_dim_coord():
     da1 = DataArray([1, 2, 3], dims="x", coords={"x": [1, 2, 3], "y": 1})
     da2 = DataArray([4, 5, 6], dims="x", coords={"x": [4, 5, 6]})
     da3 = DataArray([7, 8, 9], dims="x", coords={"x": [7, 8, 9], "y": 1})
-    for coords in ["different", "all"]:
-        with pytest.raises(ValueError, match=r"'y' not present in all datasets"):
-            concat([da1, da2, da3], dim="x", coords=coords)
+
+    with pytest.raises(ValueError, match=r"'y' not present in all datasets"):
+        concat([da1, da2, da3], dim="x", coords="all")
+
+    with pytest.raises(ValueError, match=r"'y' not present in all datasets"):
+        concat([da1, da2, da3], dim="x", coords="different", compat="equals")
 
 
 def test_concat_preserve_coordinate_order() -> None:
@@ -1379,3 +1439,150 @@ def test_concat_index_not_same_dim() -> None:
         match=r"Cannot concatenate along dimension 'x' indexes with dimensions.*",
     ):
         concat([ds1, ds2], dim="x")
+
+
+class TestNewDefaults:
+    def test_concat_second_empty_with_scalar_data_var_only_on_first(self) -> None:
+        ds1 = Dataset(data_vars={"a": ("y", [0.1]), "b": 0.1}, coords={"x": 0.1})
+        ds2 = Dataset(coords={"x": 0.1})
+
+        expected = Dataset(
+            data_vars={"a": ("y", [0.1, np.nan]), "b": 0.1}, coords={"x": 0.1}
+        )
+        with set_options(use_new_combine_kwarg_defaults=False):
+            with pytest.warns(
+                FutureWarning,
+                match="will change from compat='equals' to compat='override'",
+            ):
+                actual = concat(
+                    [ds1, ds2], dim="y", coords="different", data_vars="different"
+                )
+                assert_identical(actual, expected)
+        with set_options(use_new_combine_kwarg_defaults=True):
+            with pytest.raises(ValueError, match="might be related to new default"):
+                concat([ds1, ds2], dim="y", coords="different", data_vars="different")
+
+    def test_concat_multiple_datasets_missing_vars(self) -> None:
+        vars_to_drop = [
+            "temperature",
+            "pressure",
+            "humidity",
+            "precipitation",
+            "cloud_cover",
+        ]
+
+        datasets = create_concat_datasets(
+            len(vars_to_drop), seed=123, include_day=False
+        )
+        # set up the test data
+        datasets = [
+            ds.drop_vars(varname)
+            for ds, varname in zip(datasets, vars_to_drop, strict=True)
+        ]
+        with set_options(use_new_combine_kwarg_defaults=False):
+            old = concat(datasets, dim="day")
+        with set_options(use_new_combine_kwarg_defaults=True):
+            new = concat(datasets, dim="day")
+        assert_identical(old, new)
+
+    @pytest.mark.parametrize("coords", ["different", "minimal", "all"])
+    def test_concat_coords_kwarg(
+        self, coords: Literal["all", "minimal", "different"]
+    ) -> None:
+        data = create_test_data().drop_dims("dim3")
+
+        # make sure the coords argument behaves as expected
+        data.coords["extra"] = ("dim4", np.arange(3))
+        datasets = [g for _, g in data.groupby("dim1")]
+
+        with set_options(use_new_combine_kwarg_defaults=False):
+            expectation: AbstractContextManager = (
+                pytest.warns(
+                    FutureWarning,
+                    match="will change from compat='equals' to compat='override'",
+                )
+                if coords == "different"
+                else nullcontext()
+            )
+            with expectation:
+                old = concat(datasets, data["dim1"], coords=coords)
+
+        with set_options(use_new_combine_kwarg_defaults=True):
+            if coords == "different":
+                with pytest.raises(ValueError):
+                    concat(datasets, data["dim1"], coords=coords)
+            else:
+                new = concat(datasets, data["dim1"], coords=coords)
+                assert_identical(old, new)
+
+    def test_concat_promote_shape_for_scalars_with_mixed_lengths_along_concat_dim(
+        self,
+    ) -> None:
+        # values should repeat
+        objs = [Dataset({"x": [0]}, {"y": -1}), Dataset({"x": [1, 2]}, {"y": -2})]
+        expected = Dataset({"x": [0, 1, 2]}, {"y": ("x", [-1, -2, -2])})
+        with set_options(use_new_combine_kwarg_defaults=False):
+            with pytest.warns(
+                FutureWarning,
+                match="will change from coords='different' to coords='minimal'",
+            ):
+                old = concat(objs, "x")
+                assert_identical(old, expected)
+        with set_options(use_new_combine_kwarg_defaults=True):
+            new = concat(objs, "x")
+            with pytest.raises(AssertionError):
+                assert_identical(new, old)
+            with pytest.raises(ValueError, match="might be related to new default"):
+                concat(objs, "x", coords="different")
+            with pytest.raises(merge.MergeError, match="conflicting values"):
+                concat(objs, "x", compat="equals")
+
+            new = concat(objs, "x", coords="different", compat="equals")
+            assert_identical(old, new)
+
+
+def test_concat_multi_dim_index() -> None:
+    ds1 = (
+        Dataset(
+            {"foo": (("x", "y"), np.random.randn(2, 2))},
+            coords={"x": [1, 2], "y": [3, 4]},
+        )
+        .drop_indexes(["x", "y"])
+        .set_xindex(["x", "y"], XYIndex)
+    )
+    ds2 = (
+        Dataset(
+            {"foo": (("x", "y"), np.random.randn(2, 2))},
+            coords={"x": [1, 2], "y": [5, 6]},
+        )
+        .drop_indexes(["x", "y"])
+        .set_xindex(["x", "y"], XYIndex)
+    )
+
+    expected = (
+        Dataset(
+            {
+                "foo": (
+                    ("x", "y"),
+                    np.concatenate([ds1.foo.data, ds2.foo.data], axis=-1),
+                )
+            },
+            coords={"x": [1, 2], "y": [3, 4, 5, 6]},
+        )
+        .drop_indexes(["x", "y"])
+        .set_xindex(["x", "y"], XYIndex)
+    )
+    # note: missing 'override'
+    joins: list[types.JoinOptions] = ["inner", "outer", "exact", "left", "right"]
+    for join in joins:
+        actual = concat([ds1, ds2], dim="y", join=join)
+        assert_identical(actual, expected, check_default_indexes=False)
+
+    with pytest.raises(AlignmentError):
+        actual = concat([ds1, ds2], dim="x", join="exact")
+
+    # TODO: fix these, or raise better error message
+    with pytest.raises(AssertionError):
+        joins_lr: list[types.JoinOptions] = ["left", "right"]
+        for join in joins_lr:
+            actual = concat([ds1, ds2], dim="x", join=join)
