@@ -1,20 +1,24 @@
 from __future__ import annotations
 
+from importlib.metadata import EntryPoint
 from typing import Any
 
 import numpy as np
 import pytest
 
+from xarray import set_options
 from xarray.core.types import T_Chunks, T_DuckArray, T_NormalizedChunks
 from xarray.namedarray._typing import _Chunks
 from xarray.namedarray.daskmanager import DaskManager
 from xarray.namedarray.parallelcompat import (
+    KNOWN_CHUNKMANAGERS,
     ChunkManagerEntrypoint,
     get_chunked_array_type,
     guess_chunkmanager,
     list_chunkmanagers,
+    load_chunkmanagers,
 )
-from xarray.tests import has_dask, requires_dask
+from xarray.tests import requires_dask
 
 
 class DummyChunkedArray(np.ndarray):
@@ -44,7 +48,7 @@ class DummyChunkedArray(np.ndarray):
     def __array_finalize__(self, obj):
         if obj is None:
             return
-        self.chunks = getattr(obj, "chunks", None)
+        self.chunks = getattr(obj, "chunks", None)  # type: ignore[assignment]
 
     def rechunk(self, chunks, **kwargs):
         copied = self.copy()
@@ -86,7 +90,7 @@ class DummyChunkManager(ChunkManagerEntrypoint):
     def rechunk(self, data: DummyChunkedArray, chunks, **kwargs) -> DummyChunkedArray:
         return data.rechunk(chunks, **kwargs)
 
-    def compute(self, *data: DummyChunkedArray, **kwargs) -> tuple[np.ndarray, ...]:
+    def compute(self, *data: DummyChunkedArray, **kwargs) -> tuple[np.ndarray, ...]:  # type: ignore[override]
         from dask.array import compute
 
         return compute(*data, **kwargs)
@@ -150,8 +154,24 @@ class TestGetChunkManager:
         chunkmanager = guess_chunkmanager("dummy")
         assert isinstance(chunkmanager, DummyChunkManager)
 
-    def test_fail_on_nonexistent_chunkmanager(self) -> None:
-        with pytest.raises(ValueError, match="unrecognized chunk manager foo"):
+    def test_get_chunkmanger_via_set_options(self, register_dummy_chunkmanager) -> None:
+        with set_options(chunk_manager="dummy"):
+            chunkmanager = guess_chunkmanager(None)
+            assert isinstance(chunkmanager, DummyChunkManager)
+
+    def test_fail_on_known_but_missing_chunkmanager(
+        self, register_dummy_chunkmanager, monkeypatch
+    ) -> None:
+        monkeypatch.setitem(KNOWN_CHUNKMANAGERS, "test", "test-package")
+        with pytest.raises(
+            ImportError, match=r"chunk manager 'test' is not available.+test-package"
+        ):
+            guess_chunkmanager("test")
+
+    def test_fail_on_nonexistent_chunkmanager(
+        self, register_dummy_chunkmanager
+    ) -> None:
+        with pytest.raises(ValueError, match="unrecognized chunk manager 'foo'"):
             guess_chunkmanager("foo")
 
     @requires_dask
@@ -159,9 +179,16 @@ class TestGetChunkManager:
         chunkmanager = guess_chunkmanager(None)
         assert isinstance(chunkmanager, DaskManager)
 
-    @pytest.mark.skipif(has_dask, reason="requires dask not to be installed")
-    def test_dont_get_dask_if_not_installed(self) -> None:
-        with pytest.raises(ValueError, match="unrecognized chunk manager dask"):
+    def test_no_chunk_manager_available(self, monkeypatch) -> None:
+        monkeypatch.setattr("xarray.namedarray.parallelcompat.list_chunkmanagers", dict)
+        with pytest.raises(ImportError, match="no chunk managers available"):
+            guess_chunkmanager("foo")
+
+    def test_no_chunk_manager_available_but_known_manager_requested(
+        self, monkeypatch
+    ) -> None:
+        monkeypatch.setattr("xarray.namedarray.parallelcompat.list_chunkmanagers", dict)
+        with pytest.raises(ImportError, match="chunk manager 'dask' is not available"):
             guess_chunkmanager("dask")
 
     @requires_dask
@@ -218,3 +245,13 @@ class TestGetChunkedArrayType:
 
         with pytest.raises(TypeError, match="received multiple types"):
             get_chunked_array_type(*[dask_arr, dummy_arr])
+
+
+def test_bogus_entrypoint() -> None:
+    # Create a bogus entry-point as if the user broke their setup.cfg
+    # or is actively developing their new chunk manager
+    entry_point = EntryPoint(
+        "bogus", "xarray.bogus.doesnotwork", "xarray.chunkmanagers"
+    )
+    with pytest.warns(UserWarning, match="Failed to load chunk manager"):
+        assert len(load_chunkmanagers([entry_point])) == 0
