@@ -9,7 +9,9 @@ import pytest
 
 from xarray.core import duck_array_ops, utils
 from xarray.core.utils import (
+    attempt_import,
     either_dict_or_kwargs,
+    flat_items,
     infix_dims,
     iterate_nested,
 )
@@ -22,7 +24,7 @@ class TestAlias:
             pass
 
         old_method = utils.alias(new_method, "old_method")
-        assert "deprecated" in old_method.__doc__
+        assert "deprecated" in old_method.__doc__  # type: ignore[operator]
         with pytest.warns(Warning, match="deprecated"):
             old_method()
 
@@ -101,10 +103,10 @@ class TestDictionaries:
             utils.compat_dict_union(self.x, self.z)
 
     def test_dict_equiv(self):
-        x = {}
+        x: dict = {}
         x["a"] = 3
         x["b"] = np.array([1, 2, 3])
-        y = {}
+        y: dict = {}
         y["b"] = np.array([1.0, 2.0, 3.0])
         y["a"] = 3
         assert utils.dict_equiv(x, y)  # two nparrays are equal
@@ -128,17 +130,34 @@ class TestDictionaries:
     def test_frozen(self):
         x = utils.Frozen(self.x)
         with pytest.raises(TypeError):
-            x["foo"] = "bar"
+            x["foo"] = "bar"  # type: ignore[index]
         with pytest.raises(TypeError):
-            del x["a"]
+            del x["a"]  # type: ignore[attr-defined]
         with pytest.raises(AttributeError):
-            x.update(self.y)
+            x.update(self.y)  # type: ignore[attr-defined]
         assert x.mapping == self.x
         assert repr(x) in (
             "Frozen({'a': 'A', 'b': 'B'})",
             "Frozen({'b': 'B', 'a': 'A'})",
         )
 
+    def test_filtered(self):
+        x = utils.FilteredMapping(keys={"a"}, mapping={"a": 1, "b": 2})
+        assert "a" in x
+        assert "b" not in x
+        assert x["a"] == 1
+        assert list(x) == ["a"]
+        assert len(x) == 1
+        assert repr(x) == "FilteredMapping(keys={'a'}, mapping={'a': 1, 'b': 2})"
+        assert dict(x) == {"a": 1}
+
+
+def test_flat_items() -> None:
+    mapping = {"x": {"y": 1, "z": 2}, "x/y": 3}
+    actual = list(flat_items(mapping))
+    expected = [("x/y", 1), ("x/z", 2), ("x/y", 3)]
+    assert actual == expected
+
 
 def test_repr_object():
     obj = utils.ReprObject("foo")
@@ -220,11 +239,11 @@ def test_hidden_key_dict():
 
 
 def test_either_dict_or_kwargs():
-    result = either_dict_or_kwargs(dict(a=1), None, "foo")
+    result = either_dict_or_kwargs(dict(a=1), {}, "foo")
     expected = dict(a=1)
     assert result == expected
 
-    result = either_dict_or_kwargs(None, dict(a=1), "foo")
+    result = either_dict_or_kwargs({}, dict(a=1), "foo")
     expected = dict(a=1)
     assert result == expected
 
@@ -273,25 +292,25 @@ def test_infix_dims_errors(supplied, all_):
         pytest.param(..., ..., id="ellipsis"),
     ],
 )
-def test_parse_dims(dim, expected) -> None:
+def test_parse_dims_as_tuple(dim, expected) -> None:
     all_dims = ("a", "b", 1, ("b", "c"))  # selection of different Hashables
-    actual = utils.parse_dims(dim, all_dims, replace_none=False)
+    actual = utils.parse_dims_as_tuple(dim, all_dims, replace_none=False)
     assert actual == expected
 
 
 def test_parse_dims_set() -> None:
     all_dims = ("a", "b", 1, ("b", "c"))  # selection of different Hashables
     dim = {"a", 1}
-    actual = utils.parse_dims(dim, all_dims)
+    actual = utils.parse_dims_as_tuple(dim, all_dims)
     assert set(actual) == dim
 
 
 @pytest.mark.parametrize(
     "dim", [pytest.param(None, id="None"), pytest.param(..., id="ellipsis")]
 )
-def test_parse_dims_replace_none(dim: None | EllipsisType) -> None:
+def test_parse_dims_replace_none(dim: EllipsisType | None) -> None:
     all_dims = ("a", "b", 1, ("b", "c"))  # selection of different Hashables
-    actual = utils.parse_dims(dim, all_dims, replace_none=True)
+    actual = utils.parse_dims_as_tuple(dim, all_dims, replace_none=True)
     assert actual == all_dims
 
 
@@ -306,7 +325,7 @@ def test_parse_dims_replace_none(dim: None | EllipsisType) -> None:
 def test_parse_dims_raises(dim) -> None:
     all_dims = ("a", "b", 1, ("b", "c"))  # selection of different Hashables
     with pytest.raises(ValueError, match="'x'"):
-        utils.parse_dims(dim, all_dims, check_exists=True)
+        utils.parse_dims_as_tuple(dim, all_dims, check_exists=True)
 
 
 @pytest.mark.parametrize(
@@ -361,3 +380,14 @@ def test_find_stack_level():
         return utils.find_stack_level(test_mode=True)
 
     assert f() == 3
+
+
+def test_attempt_import() -> None:
+    """Test optional dependency handling."""
+    np = attempt_import("numpy")
+    assert np.__name__ == "numpy"
+
+    with pytest.raises(ImportError, match="The foo package is required"):
+        attempt_import(module="foo")
+    with pytest.raises(ImportError, match="The foo package is required"):
+        attempt_import(module="foo.bar")
